## $Id: tests.py 8847 2012-06-29 10:21:03Z henrik $
##
## Copyright (C) 2012 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""
Tests for mandates.
"""
import tempfile
import shutil
from datetime import datetime, timedelta
from zope.interface.verify import verifyClass, verifyObject
from zope.component import createObject
from zope.component.hooks import setSite, clearSite
from waeup.kofa.app import University
from waeup.kofa.interfaces import IUserAccount
from waeup.kofa.mandates.interfaces import (
    IMandatesContainer, IMandate)
from waeup.kofa.mandates.container import MandatesContainer
from waeup.kofa.mandates.mandate import Mandate
from waeup.kofa.testing import (FunctionalLayer, FunctionalTestCase)

class MandatesContainerTestCase(FunctionalTestCase):

    layer = FunctionalLayer

    def test_interfaces(self):
        # Make sure the correct interfaces are implemented.
        self.assertTrue(
            verifyClass(
                IMandatesContainer, MandatesContainer)
            )
        self.assertTrue(
            verifyObject(
                IMandatesContainer, MandatesContainer())
            )
        self.assertTrue(
            verifyClass(
                IMandate, Mandate)
            )
        self.assertTrue(
            verifyObject(
                IMandate, Mandate())
            )
        return

    def setUp(self):
        super(MandatesContainerTestCase, self).setUp()

        # Setup a sample site for each test
        app = University()
        self.dc_root = tempfile.mkdtemp()
        app['datacenter'].setStoragePath(self.dc_root)

        # Prepopulate the ZODB...
        self.getRootFolder()['app'] = app
        # we add the site immediately after creation to the
        # ZODB. Catalogs and other local utilities are not setup
        # before that step.
        self.app = self.getRootFolder()['app']
        # Set site here. Some of the following setup code might need
        # to access grok.getSite() and should get our new app then
        setSite(app)

    def tearDown(self):
        super(MandatesContainerTestCase, self).tearDown()
        clearSite()
        shutil.rmtree(self.dc_root)

    def test_set_password(self):
        student = createObject('waeup.Student')
        self.app['students'].addStudent(student)
        # Add and execute a mandate with missing parameters.
        mandate = Mandate()
        mandate.category = 'student_password'
        self.app['mandates'].addMandate(mandate)
        msg = mandate.execute()
        self.assertEqual(msg, u'Wrong mandate parameters.')
        # Add and execute an expired mandate.
        mandate = Mandate(days=0)
        mandate.category = 'student_password'
        self.app['mandates'].addMandate(mandate)
        msg = mandate.execute()
        self.assertEqual(msg, u'Mandate expired.')
        # Ad and execute a perfect mandate
        mandate = Mandate(days=0)
        mandate.category = 'student_password'
        mandate.params['student_id'] = student.student_id
        mandate.params['password'] = 'mypwd1'
        self.app['mandates'].addMandate(mandate)
        msg = mandate.execute()
        # Password has been set.
        self.assertEqual(msg, u'Password has been successfully set.')
        self.assertTrue(IUserAccount(student).checkPassword('mypwd1'))
        # All mandates have been removed.
        self.assertEqual(len(self.app['mandates'].keys()), 0)

    def test_remove_expired(self):
        # mandate1 is an old mandate which just expired.
        mandate1 = Mandate(days=0)
        self.app['mandates'].addMandate(mandate1)
        # mandate2 is a new mandate with default time delta.
        mandate2 = Mandate(mandate_id='23456')
        self.app['mandates'].addMandate(mandate2)
        self.assertEqual(len(self.app['mandates'].keys()), 2)
        self.app['mandates'].removeExpired()
        # Only the new mandate remains in the container.
        self.assertEqual(len(self.app['mandates'].keys()), 1)
        self.assertEqual([i for i in self.app['mandates'].keys()], [u'23456'])
